package com.crcdata.tin;

import tinfour.common.IIncrementalTin;
import tinfour.common.IQuadEdge;
import tinfour.common.Vertex;
import tinfour.standard.IncrementalTin;

import java.util.*;

public class TestTIN_36 {
    public static void main(String[] args) {
        List<Vertex> vertices = new ArrayList<>(292);
        IIncrementalTin tin = new IncrementalTin(10.0);

        vertices.add(new Vertex(35.67398328736784, 159.48210926578278, 0.0));
        vertices.add(new Vertex(38.03875619208334, 157.6644752501997, 0.0));
        vertices.add(new Vertex(37.21600267644298, 160.4848525930816, 0.0));
        vertices.add(new Vertex(34.00562309702486, 160.76445998375877, 0.0));

        vertices.add(new Vertex(19.146512421281646, 172.18560953756503, 0.0));

        for (int index = 0; index < vertices.size(); index++) {
            vertices.get(index).setIndex(index);
        }

        tin.add(vertices.get(0));
        validateEdges(tin);
        tin.add(vertices.get(1));
        validateEdges(tin);
        tin.add(vertices.get(2));
        validateEdges(tin);
        tin.add(vertices.get(3));
        validateEdges(tin);
        tin.add(vertices.get(4));
        validateEdges(tin);

    }

    private static void validateEdges(final IIncrementalTin tin) {
        Set<Vertex> vertexSet = new HashSet<>();
        for (IQuadEdge edge : tin.getEdges()) {
            vertexSet.add(edge.getA());
            vertexSet.add(edge.getB());
        }
        vertexSet.remove(null);

        for (IQuadEdge edge : tin.getEdges()) {
            for (Vertex vertex : vertexSet) {
                double dist = distance(edge, vertex);
                if (dist < 1e-10) {
                    if (edge.getA() == vertex || edge.getB() == vertex) continue;
                    System.err.printf("!!! ERROR !!! Vertex %s is near to edge: %s, distance=%s%n", vertex, edge, dist);
                    break;
                }
            }
        }
    }

    private static double distance(final IQuadEdge edge, final Vertex vertex) {
        Objects.requireNonNull(edge, "edge");
        Objects.requireNonNull(vertex, "vertex");
        if (edge.getA() == null || edge.getB() == null) return Double.NaN;

        return distancePointLine(vertex, edge.getA(), edge.getB());
    }

    private static double distancePointLine(Vertex p, Vertex A, Vertex B) {
        if (A.x == B.x && A.y == B.y) {
            return p.getDistance(A);
        } else {
            double len2 = (B.x - A.x) * (B.x - A.x) + (B.y - A.y) * (B.y - A.y);
            double r = ((p.x - A.x) * (B.x - A.x) + (p.y - A.y) * (B.y - A.y)) / len2;
            if (r <= 0.0D) {
                return Double.NaN;
            } else if (r >= 1.0D) {
                return Double.NaN;
            } else {
                double s = ((A.y - p.y) * (B.x - A.x) - (A.x - p.x) * (B.y - A.y)) / len2;
                return Math.abs(s) * Math.sqrt(len2);
            }
        }

    }
}
